@@ -1,7 +1,7 @@
 /* Metacity compositing manager */
 
 /* 
- * Copyright (C) 2003 Red Hat, Inc.
+ * Copyright (C) 2003, 2004 Red Hat, Inc.
  * Copyright (C) 2003 Keith Packard
  * 
  * This program is free software; you can redistribute it and/or
@@ -32,6 +32,10 @@
 #include <X11/extensions/Xdamage.h>
 #include <X11/extensions/Xrender.h>
 
+#if COMPOSITE_MAJOR > 0 || COMPOSITE_MINOR >= 2
+#define HAVE_NAME_WINDOW_PIXMAP 1
+#endif
+
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
 
 #define SHADOW_OFFSET 3
@@ -62,13 +66,17 @@ typedef struct
   
   Picture         picture;
   XserverRegion   border_size;
+
+#if HAVE_NAME_WINDOW_PIXMAP
+  Pixmap          pixmap;
+#endif
   
   unsigned int managed : 1;
   unsigned int damaged : 1;
   unsigned int viewable : 1;
 
   unsigned int screen_index : 8;
-  
+
 #endif  
 } MetaCompositorWindow;
 
@@ -82,8 +90,6 @@ struct MetaCompositor
   int damage_event_base;
   int fixes_error_base;
   int fixes_event_base;
-  int render_error_base;
-  int render_event_base;
   
   GHashTable *window_hash;
 
@@ -94,7 +100,9 @@ struct MetaCompositor
   guint have_composite : 1;
   guint have_damage : 1;
   guint have_fixes : 1;
-  guint have_render : 1;
+#ifdef HAVE_NAME_WINDOW_PIXMAP
+  guint have_name_window_pixmap : 1;
+#endif
 };
 
 #ifdef HAVE_COMPOSITE_EXTENSIONS
@@ -102,13 +110,20 @@ static void
 meta_compositor_window_free (MetaCompositorWindow *cwindow)
 {
   g_assert (cwindow->damage != None);
-
+  
+  meta_error_trap_push (cwindow->compositor->display);
   /* This seems to cause an error if the window
    * is destroyed?
    */
-  meta_error_trap_push (cwindow->compositor->display);
   XDamageDestroy (cwindow->compositor->display->xdisplay,
                   cwindow->damage);
+
+#if HAVE_NAME_WINDOW_PIXMAP
+  /* Free our window pixmap name */
+  if (cwindow->pixmap != None)
+    XFreePixmap (cwindow->compositor->display->xdisplay,
+                 cwindow->pixmap);
+#endif
   meta_error_trap_pop (cwindow->compositor->display, FALSE);
   
   g_free (cwindow);
@@ -143,7 +158,21 @@ meta_compositor_new (MetaDisplay *display)
       compositor->composite_error_base = 0;
     }
   else
-    compositor->have_composite = TRUE;
+    {
+      int composite_major, composite_minor;
+      
+      compositor->have_composite = TRUE;
+
+#if HAVE_NAME_WINDOW_PIXMAP
+      XCompositeQueryVersion (display->xdisplay,
+                              &composite_major, &composite_minor);
+
+#if 0
+      if (composite_major > 0 || composite_minor >= 2)
+	compositor->have_name_window_pixmap = TRUE;
+#endif
+#endif
+    }
 
   meta_topic (META_DEBUG_COMPOSITOR, "Composite extension event base %d error base %d\n",
               compositor->composite_event_base,
@@ -176,25 +205,11 @@ meta_compositor_new (MetaDisplay *display)
   meta_topic (META_DEBUG_COMPOSITOR, "Fixes extension event base %d error base %d\n",
               compositor->fixes_event_base,
               compositor->fixes_error_base);
-
-  if (!XRenderQueryExtension (display->xdisplay,
-                              &compositor->render_event_base,
-                              &compositor->render_error_base))
-    {
-      compositor->render_event_base = 0;
-      compositor->render_error_base = 0;
-    }
-  else
-    compositor->have_render = TRUE;
-
-  meta_topic (META_DEBUG_COMPOSITOR, "Render extension event base %d error base %d\n",
-              compositor->render_event_base,
-              compositor->render_error_base);
   
   if (!(compositor->have_composite &&
         compositor->have_fixes &&
-        compositor->have_render &&
-        compositor->have_damage))
+        compositor->have_damage &&
+        META_DISPLAY_HAS_RENDER (compositor->display)))
     {
       meta_topic (META_DEBUG_COMPOSITOR, "Failed to find all extensions needed for compositing manager, disabling compositing manager\n");
       g_assert (!compositor->enabled);
@@ -270,6 +285,31 @@ window_extents (MetaCompositorWindow *cwindow)
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
 
 #ifdef HAVE_COMPOSITE_EXTENSIONS
+static void
+window_get_paint_bounds (MetaCompositorWindow *cwindow,
+                         int *x,
+                         int *y,
+                         int *w,
+                         int *h)
+{
+#ifdef HAVE_NAME_WINDOW_PIXMAP
+  if (cwindow->pixmap != None)
+    {
+      *x = cwindow->x;
+      *y = cwindow->y;
+      *w = cwindow->width + cwindow->border_width * 2;
+      *h = cwindow->height + cwindow->border_width * 2;
+    }
+  else
+#endif
+    {
+      *x = cwindow->x + cwindow->border_width;
+      *y = cwindow->y + cwindow->border_width;
+      *w = cwindow->width;
+      *h = cwindow->height;
+    }
+}
+
 static void
 paint_screen (MetaCompositor *compositor,
               MetaScreen     *screen,
@@ -286,7 +326,7 @@ paint_screen (MetaCompositor *compositor,
   meta_topic (META_DEBUG_COMPOSITOR, "Repainting screen %d root 0x%lx\n",
               screen->number, screen->xroot);
 
-  meta_display_grab (screen->display);
+  /* meta_display_grab (screen->display); */
   
   xdisplay = screen->display->xdisplay;
   
@@ -378,20 +418,24 @@ paint_screen (MetaCompositor *compositor,
            meta_grab_op_is_moving (compositor->display->grab_op)))
         {
           /* Draw window transparent while resizing */
+          int x, y, w, h;
+
+          window_get_paint_bounds (cwindow, &x, &y, &w, &h);
+          
           XRenderComposite (xdisplay,
                             PictOpOver, /* PictOpOver for alpha, PictOpSrc without */
                             cwindow->picture,
                             screen->trans_picture,
                             buffer_picture,
                             0, 0, 0, 0,
-                            cwindow->x + cwindow->border_width,
-                            cwindow->y + cwindow->border_width,
-                            cwindow->width,
-                            cwindow->height);
+                            x, y, w, h);
         }
       else
         {
           /* Draw window normally */
+          int x, y, w, h;
+          
+          window_get_paint_bounds (cwindow, &x, &y, &w, &h);
           
           /* superlame drop shadow */
           XRenderFillRectangle (xdisplay, PictOpOver,
@@ -407,10 +451,7 @@ paint_screen (MetaCompositor *compositor,
                             None,
                             buffer_picture,
                             0, 0, 0, 0,
-                            cwindow->x + cwindow->border_width,
-                            cwindow->y + cwindow->border_width,
-                            cwindow->width,
-                            cwindow->height);
+                            x, y, w, h);
         }
 
     next:
@@ -439,7 +480,7 @@ paint_screen (MetaCompositor *compositor,
   XRenderFreePicture (xdisplay, buffer_picture);
   XFreeGC (xdisplay, gc);
 
-  meta_display_ungrab (screen->display);
+  /* meta_display_ungrab (screen->display); */
 
   XSync (screen->display->xdisplay, False);
 }
@@ -1088,18 +1129,35 @@ meta_compositor_add_window (MetaCompositor    *compositor,
   cwindow->height = attrs->height;
   cwindow->border_width = attrs->border_width;
 
+#if HAVE_NAME_WINDOW_PIXMAP
+  if (compositor->have_name_window_pixmap)
+    {
+      meta_error_trap_push (compositor->display);
+      cwindow->pixmap = XCompositeNameWindowPixmap (compositor->display->xdisplay,
+                                                    cwindow->xwindow);
+      meta_error_trap_pop (compositor->display, FALSE);
+    }
+#endif
+  
   /* viewable == mapped for the root window, since root can't be unmapped */
   cwindow->viewable = (attrs->map_state == IsViewable);
+  /* FIXME this assertion can fail somehow... */
   g_assert (attrs->map_state != IsUnviewable);
   
   pa.subwindow_mode = IncludeInferiors;
 
   if (attrs->class != InputOnly)
-    {
+    {      
       format = XRenderFindVisualFormat (compositor->display->xdisplay,
                                         attrs->visual);
       cwindow->picture = XRenderCreatePicture (compositor->display->xdisplay,
-                                               xwindow,
+#if HAVE_NAME_WINDOW_PIXMAP
+                                               cwindow->pixmap != None ?
+                                               cwindow->pixmap :
+                                               cwindow->xwindow,
+#else
+                                               cwindow->xwindow,
+#endif
                                                format,
                                                CPSubwindowMode,
                                                &pa);
@@ -1299,7 +1357,3 @@ meta_compositor_damage_window (MetaCompositor *compositor,
                                    window_extents (cwindow));
 #endif /* HAVE_COMPOSITE_EXTENSIONS */
 }
-
-
-
-
